"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _events = _interopRequireDefault(require("events"));

var _http = _interopRequireDefault(require("http"));

var _https = _interopRequireDefault(require("https"));

var _semver = _interopRequireDefault(require("semver"));

var _Logger = _interopRequireDefault(require("../Logger"));

var _classes = require("../classes");

var _errors = require("../errors");

var _factories = require("../factories");

var _utilities = require("../utilities");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const log = _Logger.default.child({
  namespace: 'bootstrap'
});

const createConfiguration = configurationInput => {
  // eslint-disable-next-line no-process-env
  const DEFAULT_ENVIRONMENT_VARIABLE_NAMESPACE = typeof process.env.GLOBAL_AGENT_ENVIRONMENT_VARIABLE_NAMESPACE === 'string' ? process.env.GLOBAL_AGENT_ENVIRONMENT_VARIABLE_NAMESPACE : 'GLOBAL_AGENT_';
  return { ...configurationInput,
    environmentVariableNamespace: typeof configurationInput.environmentVariableNamespace === 'string' ? configurationInput.environmentVariableNamespace : DEFAULT_ENVIRONMENT_VARIABLE_NAMESPACE
  };
};

const bootstrap = configurationInput => {
  const configuration = createConfiguration(configurationInput);
  global.GLOBAL_AGENT = global.GLOBAL_AGENT || (0, _factories.createGlobalAgentGlobal)();

  if (global.GLOBAL_AGENT.bootstrapped) {
    log.warn('found global.globalAgent; second attempt to bootstrap global-agent was ignored');
    return;
  }

  global.GLOBAL_AGENT.bootstrapped = true; // eslint-disable-next-line no-process-env

  global.GLOBAL_AGENT.HTTP_PROXY = process.env[configuration.environmentVariableNamespace + 'HTTP_PROXY'] || null; // eslint-disable-next-line no-process-env

  global.GLOBAL_AGENT.HTTPS_PROXY = process.env[configuration.environmentVariableNamespace + 'HTTPS_PROXY'] || null; // eslint-disable-next-line no-process-env

  global.GLOBAL_AGENT.NO_PROXY = process.env[configuration.environmentVariableNamespace + 'NO_PROXY'] || null;
  log.info({
    configuration: global.GLOBAL_AGENT
  }, 'global agent has been initialized');

  const isProxyConfigured = getProxy => {
    return () => {
      return getProxy();
    };
  };

  const mustUrlUseProxy = getProxy => {
    return url => {
      if (!getProxy()) {
        return false;
      }

      if (!global.GLOBAL_AGENT.NO_PROXY) {
        return true;
      }

      return !(0, _utilities.isUrlMatchingNoProxy)(url, global.GLOBAL_AGENT.NO_PROXY);
    };
  };

  const getUrlProxy = getProxy => {
    return () => {
      const proxy = getProxy();

      if (!proxy) {
        throw new _errors.UnexpectedStateError('HTTP(S) proxy must be configured.');
      }

      return (0, _utilities.parseProxyUrl)(proxy);
    };
  };

  const eventEmitter = new _events.default();

  const getHttpProxy = () => {
    return global.GLOBAL_AGENT.HTTP_PROXY;
  };

  const httpAgent = new _classes.HttpProxyAgent(isProxyConfigured(getHttpProxy), mustUrlUseProxy(getHttpProxy), getUrlProxy(getHttpProxy), _http.default.globalAgent, eventEmitter);

  const getHttpsProxy = () => {
    return global.GLOBAL_AGENT.HTTPS_PROXY || global.GLOBAL_AGENT.HTTP_PROXY;
  };

  const httpsAgent = new _classes.HttpsProxyAgent(isProxyConfigured(getHttpsProxy), mustUrlUseProxy(getHttpsProxy), getUrlProxy(getHttpsProxy), _https.default.globalAgent, eventEmitter); // Overriding globalAgent was added in v11.7.
  // @see https://nodejs.org/uk/blog/release/v11.7.0/

  if (_semver.default.gte(process.version, 'v11.7.0')) {
    // @see https://github.com/facebook/flow/issues/7670
    // $FlowFixMe
    _http.default.globalAgent = httpAgent; // $FlowFixMe

    _https.default.globalAgent = httpsAgent;
  } else {
    // $FlowFixMe
    _http.default.get = (0, _utilities.bindHttpMethod)(_http.default.get, httpAgent); // $FlowFixMe

    _http.default.request = (0, _utilities.bindHttpMethod)(_http.default.request, httpAgent); // $FlowFixMe

    _https.default.get = (0, _utilities.bindHttpMethod)(_https.default.get, httpsAgent); // $FlowFixMe

    _https.default.request = (0, _utilities.bindHttpMethod)(_https.default.request, httpsAgent);
  }
};

var _default = bootstrap;
exports.default = _default;
//# sourceMappingURL=bootstrap.js.map